@@ -25,7 +25,7 @@
  * list of people on the gedit Team.  
  * See the ChangeLog files for a list of changes.
  *
- * $Id: gedit-close-confirmation-dialog.c,v 1.16 2006/06/15 14:09:56 sfre Exp $
+ * $Id: gedit-close-confirmation-dialog.c,v 1.17 2006/07/17 15:54:49 jessevdk Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -679,16 +679,16 @@ build_multiple_docs_dialog (GeditCloseConfirmationDialog *dlg)
 		str = g_strdup_printf (
 				ngettext ("Changes to %d document will be permanently lost.",
 					  "Changes to %d documents will be permanently lost.",
-					  g_slist_length (priv->unsaved_documents)),
-				g_slist_length (priv->unsaved_documents));
+					  g_list_length (priv->unsaved_documents)),
+				g_list_length (priv->unsaved_documents));
 	else
 		str = g_strdup_printf (
 				ngettext ("There is %d document with unsaved changes. "
 					  "Save changes before closing?",
 					  "There are %d documents with unsaved changes. "
 					  "Save changes before closing?",
-					  g_slist_length (priv->unsaved_documents)),
-				g_slist_length (priv->unsaved_documents));
+					  g_list_length (priv->unsaved_documents)),
+				g_list_length (priv->unsaved_documents));
 
 	markup_str = g_strconcat ("<span weight=\"bold\" size=\"larger\">", str, "</span>", NULL);
 	g_free (str);
