@@ -27,7 +27,7 @@
  * list of people on the gedit Team.
  * See the ChangeLog files for a list of changes.
  *
- * $Id: gedit-commands-file.c,v 1.7 2006/04/20 17:01:13 paolo Exp $
+ * $Id: gedit-commands-file.c,v 1.12 2006/08/07 16:42:40 pborelli Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -46,7 +46,6 @@
 #include "gedit-statusbar.h"
 #include "gedit-debug.h"
 #include "gedit-utils.h"
-#include "gedit-recent.h"
 #include "dialogs/gedit-close-confirmation-dialog.h"
 #include "dialogs/gedit-open-location-dialog.h"
 #include "gedit-file-chooser-dialog.h"
@@ -67,14 +66,57 @@ static void tab_state_changed_while_saving (GeditTab    *tab,
 					    GeditWindow *window);
 
 void
-gedit_cmd_file_new (GtkAction   *action,
-		    GeditWindow *window)
+_gedit_cmd_file_new (GtkAction   *action,
+		     GeditWindow *window)
 {
 	gedit_debug (DEBUG_COMMANDS);
 
 	gedit_window_create_tab (window, TRUE);
 }
 
+static GeditTab *
+get_tab_from_uri (const GList *docs, 
+		  const gchar *uri)
+{
+	while (docs != NULL)
+	{
+		gchar *u;
+		GeditDocument *d;
+		
+		d = GEDIT_DOCUMENT (docs->data);
+
+		u = gedit_document_get_uri (d);
+
+		if ((u != NULL) && gnome_vfs_uris_match (uri, u))
+		{
+			g_free (u);
+			
+			return gedit_tab_get_from_document (d);
+		}
+		
+		g_free (u);
+		
+		docs = g_list_next (docs);
+	}
+	
+	return NULL;
+}
+
+static gboolean
+is_duplicated_uri (const GSList *uris, 
+		   const gchar  *u)
+{
+	while (uris != NULL)
+	{
+		if (gnome_vfs_uris_match (u, (const gchar*)uris->data))
+			return TRUE;
+			
+		uris = g_slist_next (uris);
+	}
+	
+	return FALSE;
+}
+
 /* File loading */
 static gint
 load_file_list (GeditWindow         *window,
@@ -89,10 +131,50 @@ load_file_list (GeditWindow         *window,
 	gboolean       jump_to = TRUE; /* Whether to jump to the new tab */
 	gboolean       flash = TRUE;   /* Whether to flash a message in the statusbar */
 
+	GList         *win_docs;
+	GSList        *uris_to_load = NULL;
+	const GSList  *l;
+	
 	gedit_debug (DEBUG_COMMANDS);
 
 	g_return_val_if_fail ((uris != NULL) && (uris->data != NULL), 0);
 
+	win_docs = gedit_window_get_documents (window);
+	
+	/* Remove the uris corresponding to documents already open in "window".
+	   Add remove duplicates from "uris" list */
+	l = uris;
+	while (uris != NULL)
+	{
+		if (!is_duplicated_uri (uris_to_load, uris->data))
+		{
+			tab = get_tab_from_uri (win_docs, (const gchar *)uris->data);
+			if (tab != NULL)
+			{
+				if (uris == l)
+				{
+					gedit_window_set_active_tab (window, tab);
+					jump_to = FALSE;
+				}
+				
+				++loaded_files;
+			}
+			else
+			{
+				uris_to_load = g_slist_prepend (uris_to_load, 
+								uris->data);
+			}
+		}
+				
+		uris = g_slist_next (uris);
+	}
+	
+	if (uris_to_load == NULL)
+		return loaded_files;
+		
+	uris_to_load = g_slist_reverse (uris_to_load);
+	l = uris_to_load;
+
 	tab = gedit_window_get_active_tab (window);
 	if (tab != NULL)
 	{
@@ -105,7 +187,7 @@ load_file_list (GeditWindow         *window,
 		{
 			const gchar *uri;
 
-			uri = (const gchar *)uris->data;
+			uri = (const gchar *)uris_to_load->data;
 
 			_gedit_tab_load (tab,
 					 uri,
@@ -113,10 +195,10 @@ load_file_list (GeditWindow         *window,
 					 line_pos,
 					 create);
 
-			uris = g_slist_next (uris);
+			uris_to_load = g_slist_next (uris_to_load);
 			jump_to = FALSE;
 
-			if (uris == NULL)
+			if (uris_to_load == NULL)
 			{
 				/* There is only a single file to load */
 				gchar *uri_for_display;
@@ -137,12 +219,12 @@ load_file_list (GeditWindow         *window,
 		}
 	}
 
-	while (uris != NULL)
+	while (uris_to_load != NULL)
 	{
-		g_return_val_if_fail (uris->data != NULL, 0);
+		g_return_val_if_fail (uris_to_load->data != NULL, 0);
 		
 		tab = gedit_window_create_tab_from_uri (window,
-							(const gchar *)uris->data,
+							(const gchar *)uris_to_load->data,
 							encoding,
 							line_pos,
 							create,
@@ -154,7 +236,7 @@ load_file_list (GeditWindow         *window,
 			++loaded_files;
 		}
 
-		uris = g_slist_next (uris);
+		uris_to_load = g_slist_next (uris_to_load);
 	}
 
 	if (flash)
@@ -186,31 +268,67 @@ load_file_list (GeditWindow         *window,
 						       loaded_files);
 		}
 	}
+	
+	/* Free uris_to_load. Note that l points to the first element of uris_to_load */
+	g_slist_free ((GSList *)l);
 
 	return loaded_files;
 }
 
-/* Exported so it can be used for drag'n'drop */
-gint
-gedit_cmd_load_files (GeditWindow         *window,
-		      const GSList        *uris,
-		      const GeditEncoding *encoding)
+/**
+ * gedit_commands_load_uri:
+ *
+ * Do nothing if URI does not exist
+ */
+void
+gedit_commands_load_uri (GeditWindow         *window,
+			 const gchar         *uri,
+			 const GeditEncoding *encoding,
+			 gint                 line_pos)
 {
-	gedit_debug (DEBUG_COMMANDS);
+	GSList *uris = NULL;
+	
+	g_return_if_fail (GEDIT_IS_WINDOW (window));
+	g_return_if_fail (uri != NULL);
 
-	return load_file_list (window, uris, encoding, 0, FALSE);
+	gedit_debug (DEBUG_COMMANDS);	
+	
+	uris = g_slist_prepend (uris, (gchar *)uri);
+	
+	load_file_list (window, uris, encoding, line_pos, FALSE);
+	
+	g_slist_free (uris);	
+}
+
+/**
+ * gedit_commands_load_uri:
+ *
+ * Ignore non-existing URIs 
+ */
+gint
+gedit_commands_load_uris (GeditWindow         *window,
+			  const GSList        *uris,
+			  const GeditEncoding *encoding,
+			  gint                 line_pos)
+{	
+	g_return_val_if_fail (GEDIT_IS_WINDOW (window), 0);
+	g_return_val_if_fail ((uris != NULL) && (uris->data != NULL), 0);
+	
+	gedit_debug (DEBUG_COMMANDS);
+	
+	return load_file_list (window, uris, encoding, line_pos, FALSE);
 }
 
 /*
  * From the command line we can specify a line position for the
- * first doc. Beside specifying a not existing uri crates a
+ * first doc. Beside specifying a not existing uri creates a
  * titled document.
  */
 gint
-gedit_cmd_load_files_from_prompt (GeditWindow         *window,
-				  const GSList        *uris,
-				  const GeditEncoding *encoding,
-				  gint                 line_pos)
+_gedit_cmd_load_files_from_prompt (GeditWindow         *window,
+				   const GSList        *uris,
+				   const GeditEncoding *encoding,
+				   gint                 line_pos)
 {
 	gedit_debug (DEBUG_COMMANDS);
 
@@ -253,17 +371,18 @@ open_dialog_response_cb (GeditFileChooserDialog *dialog,
 
 	gtk_widget_destroy (GTK_WIDGET (dialog));
 
-	gedit_cmd_load_files (window,
-			      uris,
-			      encoding);
+	gedit_commands_load_uris (window,
+				  uris,
+				  encoding,
+				  0);
 
 	g_slist_foreach (uris, (GFunc) g_free, NULL);
 	g_slist_free (uris);
 }
 
 void
-gedit_cmd_file_open (GtkAction   *action,
-		     GeditWindow *window)
+_gedit_cmd_file_open (GtkAction   *action,
+		      GeditWindow *window)
 {
 	GtkWidget     *open_dialog;
 	gpointer       data;
@@ -415,17 +534,18 @@ open_location_dialog_response_cb (GeditOpenLocationDialog *dlg,
 
 	gtk_widget_destroy (GTK_WIDGET (dlg));
 
-	gedit_cmd_load_files (window,
-		 	      uris,
-			      encoding);
+	gedit_commands_load_uris (window,
+				  uris,
+				  encoding,
+				  0);
 
 	g_slist_foreach (uris, (GFunc) g_free, NULL);
 	g_slist_free (uris);
 }
 
 void
-gedit_cmd_file_open_uri (GtkAction   *action,
-			 GeditWindow *window)
+_gedit_cmd_file_open_uri (GtkAction   *action,
+			  GeditWindow *window)
 {
 	GtkWidget *dlg;
 	gpointer   data;
@@ -461,26 +581,6 @@ gedit_cmd_file_open_uri (GtkAction   *action,
 	gtk_widget_show (dlg);
 }
 
-void
-gedit_cmd_file_open_recent (EggRecentItem *item,
-			    GeditWindow   *window)
-{
-	GSList *uris = NULL;
-	gchar  *uri;
-
-	gedit_debug (DEBUG_COMMANDS);
-
-	uri = egg_recent_item_get_uri (item);
-
-	uris = g_slist_prepend (uris, uri);
-
-	if (gedit_cmd_load_files (window, uris, NULL) != 1)
-		gedit_recent_remove (uri);
-
-	g_slist_foreach (uris, (GFunc) g_free, NULL);
-	g_slist_free (uris);
-}
-
 /* File saving */
 static void file_save_as (GeditTab *tab, GeditWindow *window);
 
@@ -671,28 +771,28 @@ confirm_overwrite_callback (GtkFileChooser *dialog,
 			    gpointer        data)
 {
 	gchar *uri;
-	gboolean ro;
+	GtkFileChooserConfirmation res;
 
 	gedit_debug (DEBUG_COMMANDS);
 
 	uri = gtk_file_chooser_get_uri (dialog);
 
-	ro = is_read_only (uri);
-
-	g_free (uri);
-
-	if (ro)
+	if (is_read_only (uri))
 	{
 		if (replace_read_only_file (GTK_WINDOW (dialog), uri))
-			return GTK_FILE_CHOOSER_CONFIRMATION_ACCEPT_FILENAME;
+			res = GTK_FILE_CHOOSER_CONFIRMATION_ACCEPT_FILENAME;
 		else
-			return GTK_FILE_CHOOSER_CONFIRMATION_SELECT_AGAIN;
+			res = GTK_FILE_CHOOSER_CONFIRMATION_SELECT_AGAIN;
 	}
 	else
 	{
 		/* fall back to the default confirmation dialog */
-		return GTK_FILE_CHOOSER_CONFIRMATION_CONFIRM;
+		res = GTK_FILE_CHOOSER_CONFIRMATION_CONFIRM;
 	}
+
+	g_free (uri);
+
+	return res;
 }
 
 static void
@@ -821,7 +921,7 @@ file_save (GeditTab    *tab,
 }
 
 void
-gedit_cmd_file_save (GtkAction   *action,
+_gedit_cmd_file_save (GtkAction   *action,
 		     GeditWindow *window)
 {
 	GeditTab *tab;
@@ -836,7 +936,7 @@ gedit_cmd_file_save (GtkAction   *action,
 }
 
 void
-gedit_cmd_file_save_as (GtkAction   *action,
+_gedit_cmd_file_save_as (GtkAction   *action,
 			GeditWindow *window)
 {
 	GeditTab *tab;
@@ -962,7 +1062,7 @@ _gedit_cmd_file_save_documents_list (GeditWindow *window,
 }
 
 void
-gedit_cmd_file_save_all (GtkAction   *action,
+_gedit_cmd_file_save_all (GtkAction   *action,
 			 GeditWindow *window)
 {
 	GList *docs;
@@ -1136,7 +1236,7 @@ revert_dialog (GeditWindow   *window,
 }
 
 void
-gedit_cmd_file_revert (GtkAction   *action,
+_gedit_cmd_file_revert (GtkAction   *action,
 		       GeditWindow *window)
 {
 	GeditDocument  *doc;
@@ -1613,7 +1713,7 @@ _gedit_cmd_file_close_tab (GeditTab    *tab,
 }
 
 void
-gedit_cmd_file_close (GtkAction   *action,
+_gedit_cmd_file_close (GtkAction   *action,
 		      GeditWindow *window)
 {
 	GeditTab *active_tab;
@@ -1699,7 +1799,7 @@ file_close_all (GeditWindow *window,
 }
 
 void
-gedit_cmd_file_close_all (GtkAction   *action,
+_gedit_cmd_file_close_all (GtkAction   *action,
 			  GeditWindow *window)
 {
 	gedit_debug (DEBUG_COMMANDS);
@@ -1714,7 +1814,7 @@ gedit_cmd_file_close_all (GtkAction   *action,
 
 /* Quit */
 void
-gedit_cmd_file_quit (GtkAction   *action,
+_gedit_cmd_file_quit (GtkAction   *action,
 		     GeditWindow *window)
 {
 	gedit_debug (DEBUG_COMMANDS);
