@@ -28,14 +28,13 @@
  * list of people on the gedit Team.  
  * See the ChangeLog files for a list of changes. 
  *
- * $Id: gedit-commands.h,v 1.17 2006/06/18 09:05:20 paolo Exp $
+ * $Id: gedit-commands.h,v 1.18 2006/08/07 16:42:40 pborelli Exp $
  */
 
 #ifndef __GEDIT_COMMANDS_H__
 #define __GEDIT_COMMANDS_H__
 
 #include <gtk/gtkaction.h>
-#include "recent-files/egg-recent-item.h"
 #include "gedit-window.h"
 
 G_BEGIN_DECLS
@@ -70,8 +69,6 @@ void		_gedit_cmd_file_new			(GtkAction   *action,
 							 GeditWindow *window);
 void		_gedit_cmd_file_open			(GtkAction   *action,
 							 GeditWindow *window);
-void		_gedit_cmd_file_open_recent		(EggRecentItem *item,
-							 GeditWindow *window);
 void		_gedit_cmd_file_save			(GtkAction   *action,
 							 GeditWindow *window);
 void		_gedit_cmd_file_save_as			(GtkAction   *action,
