@@ -25,7 +25,7 @@
  * list of people on the gedit Team.  
  * See the ChangeLog files for a list of changes. 
  *
- * $Id: gedit-panel.h,v 1.3 2006/01/02 14:26:40 paolo Exp $
+ * $Id: gedit-panel.h,v 1.4 2006/01/19 10:33:30 pborelli Exp $
  */
 
 #ifndef __GEDIT_PANEL_H__
@@ -70,10 +70,21 @@ typedef struct _GeditPanelClass GeditPanelClass;
 struct _GeditPanelClass 
 {
 	GtkVBoxClass parent_class;
-	
+
+	void (* item_added)     (GeditPanel     *panel,
+				 GtkWidget      *item);
+	void (* item_removed)   (GeditPanel     *panel,
+				 GtkWidget      *item);
+
 	/* Keybinding signals */
 	void (* close)          (GeditPanel     *panel);
-	void (* focus_document) (GeditPanel     *panel);	
+	void (* focus_document) (GeditPanel     *panel);
+
+	/* Padding for future expansion */
+	void (*_gedit_reserved1) (void);
+	void (*_gedit_reserved2) (void);
+	void (*_gedit_reserved3) (void);
+	void (*_gedit_reserved4) (void);	
 };
 
 /*
@@ -104,6 +115,8 @@ gboolean	 gedit_panel_item_is_active 	(GeditPanel     *panel,
 
 GtkOrientation	 gedit_panel_get_orientation	(GeditPanel	*panel);
 
+gint		 gedit_panel_get_n_items	(GeditPanel	*panel);
+
 
 G_END_DECLS
 
